package com.superversion.rasa.service.impl;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.StrUtil;
import com.superversion.rasa.config.ThreadPoolExecutorConfig;
import com.superversion.rasa.pojo.vo.RasaArgument;
import com.superversion.rasa.service.RasaCmdService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.*;
import java.util.function.Predicate;

@Service
@Slf4j
@RequiredArgsConstructor
public class RasaCmdServiceImpl implements RasaCmdService {


    @Value("${rasa.models-path}")
    private String modelsPath;

    @Value("${rasa.endpoints}")
    private String endpoints;

    @Value("${rasa.config}")
    private String config;

    @Value("${rasa.data-path}")
    private String dataPath;


    @Value("${rasa.shell-env:/bin/bash}")
    private String shellEnv;

    @Value("${rasa.train-shell}")
    private String trainShell;

    @Value("${rasa.run-shell}")
    private String runShell;

    @Value("${rasa.shell.work:/home/rasa_manage/}")
    private String shellWork;

    @Override
    public String trainExec(RasaArgument argument) throws ExecutionException, InterruptedException, TimeoutException {

        String domain = dataPath+"domain.yml";
        List<String> cmds = ListUtil.toList(shellEnv, trainShell,config,dataPath,domain,modelsPath);
        cmds.add(argument.getFixedModelName());

        log.info("trainExec cmd : {}",StrUtil.join(" ",cmds));
        System.out.println("trainExec cmd sout:"+StrUtil.join(" ",cmds));
        return String.join("\r\n",execCmd(cmds,s->false,90));
    }



    @Override
    public String runExec(RasaArgument argument) throws ExecutionException, InterruptedException, TimeoutException {


        String mPath = modelsPath+argument.getFixedModelName()+".tar.gz";
        List<String> cmds = ListUtil.toList(shellEnv, runShell,mPath,endpoints,argument.getPort());

        log.info("runExec cmd : {}",StrUtil.join(" ",cmds));
        System.out.println("runExec cmd sout:"+StrUtil.join(" ",cmds));
        return String.join("\r\n",execCmd(cmds,s-> StrUtil.isNotBlank(s)&& s.contains("Rasa server is up and running"),90));
    }



    private List<String> execCmd(List<String> cmds, Predicate<String> endPredicate, long timeOut) throws InterruptedException, ExecutionException, TimeoutException {
        ProcessBuilder processBuilder = new ProcessBuilder(cmds);
        processBuilder.directory(new File(shellWork)); // 设置工作目录
        processBuilder.redirectErrorStream(true); // 合并标准输出和错误输出

        ThreadPoolExecutor instance = ThreadPoolExecutorConfig.getInstance();
        Future<List<String>> future = instance.submit(() -> {
            Process process = processBuilder.start(); // 启动进程
            InputStream inputStream = process.getInputStream(); // 获取进程的输出流
            Reader reader = new InputStreamReader(inputStream, "UTF-8");
            BufferedReader bufferedReader = new BufferedReader(reader);
            List<String> outString = new ArrayList<>();
            String resultLines = bufferedReader.readLine();
            while(resultLines != null) {
                resultLines = bufferedReader.readLine(); // 读取下一行
                log.info("resultLines:{}",resultLines);
                if (endPredicate.test(resultLines)){
                    break;
                }
                outString.add(resultLines);
            }
            bufferedReader.close();
            return outString;
        });

        return future.get(timeOut, TimeUnit.SECONDS);
    }

}
